import math
import time

def timed_function(func, *args, **kwargs):
    func_name = str(func).split(' ')[1]
    def new_func(*args, **kwargs):
        t = time.ticks_us()
        result = func(*args, **kwargs)
        delta = time.ticks_diff(time.ticks_us(), t)
        print('function: {} time: {:.3f}ms'.format(func_name, delta / 1000))
        return result
    return new_func


def get_abs(x):
    if x >= 0:
        return x
    else:
        return -x

def remove_noise(img, ext, c_thres):
    width = img.width()
    height = img.height()
    for x in range(width):
        for y in range(height):
            if img.get_pixel(x, y) > 128:
                count = -1
                s_x = x - ext
                s_y = y - ext
                for i in range(math.pow(1+ext, 2)):
                    n_x = s_x + i % (2*ext + 1)
                    n_y = s_y + i // (2*ext + 1)
                    if n_x >=0 and n_y >= 0 and n_x < width and n_y < height:
                        if img.get_pixel(n_x, n_y) > 128:
                            count = count + 1
                if count < c_thres:
                    img.set_pixel(x, y, 0)
    return img


class PID:
    kp = 0
    ki = 0
    kd = 0
    inte = 0
    last_err = 0

    def pid_iter(self, r, y):
        err = float(r - y)
        u = self.kp * err
        if err < 0.02:
            self.inte = self.inte + err
            u += self.ki * self.inte
        u += self.kd * (err - self.last_err)
        self.last_err = err
        return u

    def pid_cla(self):
        self.inte = 0
        self.last_err = 0
