import time
import struct
import serial
import serial.tools.list_ports

class ServoLib(object):
    port = None
    baud = 115200
    wait = 0.02
    sync_count = 0
    
    def __init__(self) -> None:
        if self.port==None:
            self.get_port()
        if self.port!=None:
            self.open()
                
    def get_port(self):
        port_list = list(serial.tools.list_ports.comports())
        if len(port_list)>1:
            print('\n'.join([ f"{i} - {port_list[i].description}" for i in range(len(port_list))]))
            port_index = -1
            while True:
                try:
                    port_index = int(input("Select Port > "))
                    if port_index>=0 and port_index<len(port_list):
                        break
                except:
                    pass
                print("Void Input")
            print(f"Use Port: {port_list[port_index].description}.")
            self.port = port_list[port_index].name
        elif len(port_list)==1:
            print(f"Use Port: {port_list[0].description}.")
            self.port = port_list[0].name
        else:
            print("No port Avliable.")
        
    def open(self):
        self.serial = serial.Serial(self.port,self.baud,timeout=self.wait)
    
    def close(self):
        self.serial.close()
        
    def sync(self):
        self.sync_count += 1
        for i in range(16):
            self.serial.write(struct.pack('>B',0))
            if len(self.serial.read(1)):
                print(f"MenIO_SYNC: {i+1}")
                return True
        raise Exception('Fail to SYNC.')
    
    def set(self,servo0,servo1,beep):
        self.serial.flush()
        self.serial.write(struct.pack('>ffB', servo0, servo1, (64 if beep else 1)))
        feedback = self.serial.read(1)
        if len(feedback)!=1:
            self.sync()
            return self.set(servo0,servo1,beep)
        else:
            state = struct.unpack('B',feedback)[0]
            # Beep Pause Reset
            return True if state&0x04 else False, False if state&0x02 else True, False if state&0x01 else True
        
    def set2(self,servo0,servo1,beep):
        beep, pause, reset = self.set(servo0,servo1,beep)
        if pause:
            while pause:
                beep, pause, reset = self.set(servo0,servo1,beep)
        return reset
            
if __name__ == '__main__':
    servo = ServoLib()
    servo.sync()
    for i in range(1000):
        target = i/10
        print(servo.set(50,80,False))
        time.sleep(0.01)