from ServoLibHost import ServoLib
import numpy as np
import time
import cv2


CENTER_AREA_RATE = [0.03,0.04]
CENTER_AREA = [0,0,0,0,0,0]
R_LAB_TH = (
    (105,135,119),
    (255,255,135)
)
G_LAB_TH = (
    (100,0,119),
    (255,120,135)
)
TARGET_VALUES = {
    'err_min':1,
    'err_th':0.4,
    'r_th':0.04,
    'r_x':0,
    'r_y':0,
    'r_err':0,
    'r_live':0,
    'r_err_x':0,
    'r_err_y':0,
    'g_state':0,
    'gr_match':0,
}
SERVO_X_PID = {
    'kp':12.0,
    'Ki':9.0,
    'kd':0.0,
    'th':2.0,
    'e1':0.0,
    'e2':0.0,
    'u':50.0,
}
SERVO_Y_PID = {
    'kp':12.0,
    'Ki':9.0,
    'kd':0.0,
    'th':2.0,
    'e1':0.0,
    'e2':0.0,
    'u':60.0
}

cap = cv2.VideoCapture(0)
ser = ServoLib()
  
def Choose_Color_LAB():
    def callback(object):
        pass
    ret, frame = cap.read()
    img = cv2.cvtColor(frame, cv2.COLOR_BGR2LAB)
    cv2.imshow("image",img)
    cv2.createTrackbar("L_min","image",50,255,callback)
    cv2.createTrackbar("L_max","image",150,255,callback)
    cv2.createTrackbar("A_min","image",0,255,callback)
    cv2.createTrackbar("A_max","image",255,255,callback)
    cv2.createTrackbar("B_min","image",0,255,callback)
    cv2.createTrackbar("B_max","image",255,255,callback)

    while(True):
        ret, frame = cap.read()
        img = cv2.cvtColor(frame, cv2.COLOR_BGR2LAB)
        cv2.imshow("image",img)
        L_min = cv2.getTrackbarPos("L_min","image",)
        A_min = cv2.getTrackbarPos("A_min","image",)
        B_min = cv2.getTrackbarPos("B_min","image",)
        L_max = cv2.getTrackbarPos("L_max","image",)
        A_max = cv2.getTrackbarPos("A_max","image",)
        B_max = cv2.getTrackbarPos("B_max","image",)
        lower_lab = np.array([L_min, A_min, B_min])
        upper_lab = np.array([L_max, A_max, B_max])
        mask = cv2.inRange(img,lower_lab,upper_lab)
        cv2.imshow("mask", mask)
        if cv2.waitKey(1) & 0XFF == 27:
            break

def update_center_area():
    global CENTER_AREA
    ret, frame = cap.read()
    shape = frame.shape
    CENTER_AREA[0] = int(shape[1]*(1-CENTER_AREA_RATE[0])/2)
    CENTER_AREA[1] = int(shape[1]*(1+CENTER_AREA_RATE[0])/2)
    CENTER_AREA[2] = int(shape[0]*(1-CENTER_AREA_RATE[1])/2)
    CENTER_AREA[3] = int(shape[0]*(1+CENTER_AREA_RATE[1])/2)
    CENTER_AREA[4] = shape[1]
    CENTER_AREA[5] = shape[0]
    
def update_servo_pos():
    global SERVO_X_PID
    global SERVO_Y_PID
    if TARGET_VALUES["r_live"]<1 and TARGET_VALUES["err_min"]<TARGET_VALUES["err_th"]:
        err_x = TARGET_VALUES["r_err_x"]
        d_x = SERVO_X_PID["kp"]*(err_x-SERVO_X_PID['e1'])
        d_x += SERVO_X_PID["Ki"]*err_x 
        d_x += SERVO_X_PID['kd']*(err_x - 2*SERVO_X_PID['e1'] - SERVO_X_PID['e2'])
        SERVO_X_PID['e2'] = SERVO_X_PID['e1']
        SERVO_X_PID['e1'] = err_x
        if d_x>SERVO_X_PID['th']:
            d_x = SERVO_X_PID['th']
        elif d_x<-SERVO_X_PID['th']:
            d_x = -SERVO_X_PID['th']
        SERVO_X_PID["u"] += d_x
        if SERVO_X_PID["u"]>100:
            SERVO_X_PID["u"] = 100
        elif SERVO_X_PID["u"]<0:
            SERVO_X_PID["u"] = 0

        err_y = TARGET_VALUES["r_err_y"]
        d_y = SERVO_Y_PID["kp"]*(err_y-SERVO_Y_PID['e1'])
        d_y += SERVO_Y_PID["Ki"]*err_y 
        d_y += SERVO_Y_PID['kd']*(err_y - 2*SERVO_Y_PID['e1'] - SERVO_Y_PID['e2'])
        SERVO_Y_PID['e2'] = SERVO_Y_PID['e1']
        SERVO_Y_PID['e1'] = err_y
        if d_y>SERVO_Y_PID['th']:
            d_y = SERVO_Y_PID['th']
        elif d_y<-SERVO_Y_PID['th']:
            d_y = -SERVO_Y_PID['th']
        SERVO_Y_PID["u"] += d_y
        if SERVO_Y_PID["u"]>100:
            SERVO_Y_PID["u"] = 100
        elif SERVO_Y_PID["u"]<0:
            SERVO_Y_PID["u"] = 0   
             
    else:
        SERVO_X_PID["e1"] = 0
        SERVO_X_PID["e2"] = 0
        SERVO_X_PID["u"] = 0 
        SERVO_Y_PID["e1"] = 0
        SERVO_Y_PID["e2"] = 0
        SERVO_Y_PID["u"] = 0 
        
    return ser.set2(SERVO_X_PID["u"],SERVO_Y_PID["u"],TARGET_VALUES["gr_match"])

def track_step(G):
    global TARGET_VALUES
    ret, frame = cap.read()
    frame = cv2.blur(frame,(2,2))
    grid_LAB = cv2.cvtColor(frame, cv2.COLOR_BGR2LAB)
    
    center_area_lab_roi = grid_LAB[CENTER_AREA[2]:CENTER_AREA[3],CENTER_AREA[0]:CENTER_AREA[1]] 
    
    r_lab_mask = cv2.inRange(grid_LAB, np.array(R_LAB_TH[0]), np.array(R_LAB_TH[1]))
    r_lab_mask = cv2.dilate(r_lab_mask,cv2.getStructuringElement(cv2.MORPH_ELLIPSE,(9,9)),iterations=1)
    g_lab_mask = cv2.inRange(center_area_lab_roi, np.array(G_LAB_TH[0]), np.array(G_LAB_TH[1]))
    
    r_contours, hierarchy = cv2.findContours(r_lab_mask,cv2.RETR_TREE,cv2.CHAIN_APPROX_NONE)
    if len(r_contours):
        TARGET_VALUES["err_min"] = 100
        for c in r_contours:
            (x, y), radius = cv2.minEnclosingCircle(c)
            err_x = (CENTER_AREA[4]/2 - x)/CENTER_AREA[4]*2
            err_y = (CENTER_AREA[5]/2 - y)/CENTER_AREA[5]*2
            err = (err_x**2 + err_y**2)**0.5
            if err < TARGET_VALUES["err_min"]:
                TARGET_VALUES["err_min"] = err
                TARGET_VALUES["r_x"] = x
                TARGET_VALUES["r_y"] = y
                TARGET_VALUES["r_live"] = 0
                TARGET_VALUES["r_err"] = err
                TARGET_VALUES["r_err_x"] = err_x
                TARGET_VALUES["r_err_y"] = err_y
            if G:
                cv2.circle(frame, (int(x),int(y)), int(radius), (0, 0, 255), 2)
        if G:
            cv2.circle(frame, (int(TARGET_VALUES["r_x"]),int(TARGET_VALUES["r_y"])), 10, (255, 0, 0), 3)
    else:
        TARGET_VALUES["r_live"] += 1
        
    g_contours, hierarchy = cv2.findContours(g_lab_mask,cv2.RETR_TREE,cv2.CHAIN_APPROX_NONE)
    if len(g_contours):
        TARGET_VALUES["g_state"] = 1
    else:
        TARGET_VALUES["g_state"] = 0
            
    if TARGET_VALUES["g_state"]<1 or TARGET_VALUES["r_err"] < TARGET_VALUES["r_th"] and TARGET_VALUES["r_live"] < 1:
        TARGET_VALUES["gr_match"] = 1
    else:
        TARGET_VALUES["gr_match"] = 0
        
    reset = update_servo_pos()
            
    if G:
        if TARGET_VALUES["gr_match"]:
            cv2.rectangle(frame,(CENTER_AREA[0],CENTER_AREA[2]),(CENTER_AREA[1],CENTER_AREA[3]),(0,255,0))
        else:
            cv2.rectangle(frame,(CENTER_AREA[0],CENTER_AREA[2]),(CENTER_AREA[1],CENTER_AREA[3]),(0,0,255))   
        cv2.imshow("Red_Tracker", frame)
        cv2.imshow("r_mask", r_lab_mask)
        cv2.imshow("g_mask", g_lab_mask)
        cv2.waitKey(1)
    return reset


# Choose_Color_LAB()

update_center_area()

while True:
    ser.set2(50,60,True)
    time.sleep(0.2)
    while ser.set2(50,60,False):
        pass
    SERVO_X_PID["e1"] = 0
    SERVO_X_PID["e2"] = 0
    SERVO_X_PID["u"] = 50
    SERVO_Y_PID["e1"] = 0
    SERVO_Y_PID["e2"] = 0
    SERVO_Y_PID["u"] = 60
    while True:
        if track_step(True):
            break
