#ifndef __PWMB_H_
#define __PWMB_H_

#include "ULibSTC32.h"

#define PWM_ADDR1 0x7efeb0
#define PWM_ADDR2 0x7efec0

// 捕获比较模式寄存器x4
//
// 输出模式(CCnS == 00)
// B7(OCnCE 输出比较n清零使能) B6-B4(OCnPM 输出比较n模式) B3(OCnPE 输出比较n预装使能) B2(OCnFE 输出比较n快速使能) B1-B0(CCnS 捕获比较n方向)
//
//      输出比较n清零使能       输出比较n模式
// 0    OCnREF不受ETRF影响      保持不变(冻结)
// 1    ETRF==1则OCnREF=0       CNTR==CCR
// 010  -                       CNTR!=CCR
// 011  -                       CNTR==CCR时翻转(翻转)
// 100  -                       1(强制高)
// 101  -                       0(强制低)
// 110  -                       CNTR<CCR(PWM模式1)
// 111  -                       CNTR>CCR(PWM模式2)
//
// 输入模式(CCnS != 00)
// B7-B4(ICnF 输入捕获n滤波器选择) B3-B2(ICnPSC 输入捕获n预分频器) B1-B0(CCnS 捕获比较n方向)
#define PWM_CCMR_(group, num) (*(unsigned char volatile far *)(PWM_ADDR2 + (num < 4 ? num : 3) + (group ? 0x28 : 0x08)))

// 捕获比较寄存器x4
// 若CCn通道为输出 CCRn有当前比较值
// 若CCn通道为输入 CCRn有上次捕获计数值
#define PWM_CCR_(group, num, hl) (*(unsigned char volatile far *)(PWM_ADDR2 + (hl ? 0 : 1) + ((num < 4 ? num : 3) * 2) + (group ? 0x35 : 0x15)))

#define PWMBNormal(pin) PWM_CCMR_(1, pin) = 0x68

#define PWMBHigh(pin) PWM_CCMR_(1, pin) = 0x58

#define PWMBLow(pin) PWM_CCMR_(1, pin) = 0x48

#define PWMBValue(pin, pValue)                        \
    PWM_CCR_(1, pin, 1) = *(unsigned char *)(pValue); \
    PWM_CCR_(1, pin, 0) = *((unsigned char *)(pValue) + 1)

#define PWMBInit(freq)   \
    PWMB_PS = 0xaa;      \
    PWMB_ENO = 0xff;     \
    PWMB_CCER1 = 0x55;   \
    PWMB_CCER2 = 0x55;   \
    PWMClkInit(1, freq); \
    PWMBNormal(0);       \
    PWMBNormal(1);       \
    PWMBNormal(2);       \
    PWMBNormal(3);       \
    PWMB_BKR |= 0x80

#define PWMBPeroid() ((unsigned short)(PWMB_ARRH << 8) | (unsigned short)PWMB_ARRL)

#endif