#include "ULibSTC32.h"

#define TaskI2CRead_Noop 0
#define TaskI2CRead_Send 1
#define TaskI2CRead_Recv 2

unsigned long SYSclk;

ULibTMRTaskList TMR0, TMR1, TMR2, TMR3, TMR4;
ULibTaskAddr M2MTaskAddr = AddrNull;
ULibTaskAddr ADCTaskAddr = AddrNull;
ULibTaskAddr SPITaskAddr = AddrNull;
ULibTaskAddr UR1TTaskAddr = AddrNull;
ULibTaskAddr UR1RTaskAddr = AddrNull;
ULibTaskAddr UR2TTaskAddr = AddrNull;
ULibTaskAddr UR2RTaskAddr = AddrNull;
ULibTaskAddr UR3TTaskAddr = AddrNull;
ULibTaskAddr UR3RTaskAddr = AddrNull;
ULibTaskAddr UR4TTaskAddr = AddrNull;
ULibTaskAddr UR4RTaskAddr = AddrNull;
ULibTaskAddr I2CTTaskAddr = AddrNull;
ULibTaskAddr I2CRTaskAddr = AddrNull;

// @brief 初始化STC32
void STC32Init(void)
{
    WTST = 0;
    EAXFR = 1;
    CKCON = 0;
    CLKInit(CLKMode_35000000);
    IAP_TPS = (SYSclk + 500000UL) / 1000000UL;
    EA = 1;
}

// @brief 初始化主时钟
// @param mode 主时钟频率
void CLKInit(CLKMode mode)
{
    MCLKOCR = 0x00;
    if (mode > 100)
    {
        MCLKOCR |= 0x80;
    }
    mode %= 100;
    if (mode > 10)
    {
        MCLKOCR |= (0x01 << ((mode / 10) < 8 ? (mode / 10) - 1 : 6));
    }
    mode %= 10;
    switch (mode)
    {
    case 0:
        SYSclk = 22118400;
        break;
    case 1:
        SYSclk = 24000000;
        break;
    case 2:
        SYSclk = 27000000;
        break;
    case 3:
        SYSclk = 30000000;
        break;
    case 4:
        SYSclk = 33117600;
        break;
    case 5:
        SYSclk = 35000000;
        break;
    case 6:
        SYSclk = 36864000;
        break;
    case 7:
        SYSclk = 40000000;
        break;
    case 8:
        SYSclk = 44236800;
        break;
    case 9:
        SYSclk = 48000000;
        break;
    }
    CLKDIV = 0x04;
    IRTRIM = (*(unsigned char volatile far *)(0x7efde0 + 11 + mode));
    VRTRIM = (*(unsigned char volatile far *)(0x7efde0 + 23 + (mode / 5)));
    IRCBAND = (2 + (mode / 5));
    CLKDIV = 0x00;
}

// @brief 初始化GPIO
// @param mode IO模式
// @param port 端口编号 0-7
// @param mask 引脚掩码 置1生效
void GIOInit(GIOMode mode, GIOPort port, GIOMask mask)
{
    switch (port)
    {
    case 0:
        BitSet(P0M0, mask, mode & 0x01);
        BitSet(P0M1, mask, mode & 0x02);
        BitSet(P0PU, mask, mode & 0x04);
        BitSet(P0PD, mask, mode & 0x08);
        BitSet(P0NCS, mask, mode & 0x10);
        BitSet(P0SR, mask, !(mode & 0x20));
        BitSet(P0DR, mask, !(mode & 0x40));
        BitSet(P0IE, mask, !(mode & 0x80));
        break;
    case 1:
        BitSet(P1M0, mask, mode & 0x01);
        BitSet(P1M1, mask, mode & 0x02);
        BitSet(P1PU, mask, mode & 0x04);
        BitSet(P1PD, mask, mode & 0x08);
        BitSet(P1NCS, mask, mode & 0x10);
        BitSet(P1SR, mask, !(mode & 0x20));
        BitSet(P1DR, mask, !(mode & 0x40));
        BitSet(P1IE, mask, !(mode & 0x80));
        break;
    case 2:
        BitSet(P2M0, mask, mode & 0x01);
        BitSet(P2M1, mask, mode & 0x02);
        BitSet(P2PU, mask, mode & 0x04);
        BitSet(P2PD, mask, mode & 0x08);
        BitSet(P2NCS, mask, mode & 0x10);
        BitSet(P2SR, mask, !(mode & 0x20));
        BitSet(P2DR, mask, !(mode & 0x40));
        BitSet(P2IE, mask, !(mode & 0x80));
        break;
    case 3:
        BitSet(P3M0, mask, mode & 0x01);
        BitSet(P3M1, mask, mode & 0x02);
        BitSet(P3PU, mask, mode & 0x04);
        BitSet(P3PD, mask, mode & 0x08);
        BitSet(P3NCS, mask, mode & 0x10);
        BitSet(P3SR, mask, !(mode & 0x20));
        BitSet(P3DR, mask, !(mode & 0x40));
        BitSet(P3IE, mask, !(mode & 0x80));
        break;
    case 4:
        BitSet(P4M0, mask, mode & 0x01);
        BitSet(P4M1, mask, mode & 0x02);
        BitSet(P4PU, mask, mode & 0x04);
        BitSet(P4PD, mask, mode & 0x08);
        BitSet(P4NCS, mask, mode & 0x10);
        BitSet(P4SR, mask, !(mode & 0x20));
        BitSet(P4DR, mask, !(mode & 0x40));
        BitSet(P4IE, mask, !(mode & 0x80));
        break;
    case 5:
        BitSet(P5M0, mask, mode & 0x01);
        BitSet(P5M1, mask, mode & 0x02);
        BitSet(P5PU, mask, mode & 0x04);
        BitSet(P5PD, mask, mode & 0x08);
        BitSet(P5NCS, mask, mode & 0x10);
        BitSet(P5SR, mask, !(mode & 0x20));
        BitSet(P5DR, mask, !(mode & 0x40));
        BitSet(P5IE, mask, !(mode & 0x80));
        break;
    case 6:
        BitSet(P6M0, mask, mode & 0x01);
        BitSet(P6M1, mask, mode & 0x02);
        BitSet(P6PU, mask, mode & 0x04);
        BitSet(P6PD, mask, mode & 0x08);
        BitSet(P6NCS, mask, mode & 0x10);
        BitSet(P6SR, mask, !(mode & 0x20));
        BitSet(P6DR, mask, !(mode & 0x40));
        BitSet(P6IE, mask, !(mode & 0x80));
        break;
    case 7:
        BitSet(P7M0, mask, mode & 0x01);
        BitSet(P7M1, mask, mode & 0x02);
        BitSet(P7PU, mask, mode & 0x04);
        BitSet(P7PD, mask, mode & 0x08);
        BitSet(P7NCS, mask, mode & 0x10);
        BitSet(P7SR, mask, !(mode & 0x20));
        BitSet(P7DR, mask, !(mode & 0x40));
        BitSet(P7IE, mask, !(mode & 0x80));
        break;

    default:
        break;
    }
}

// @brief 重启STC
// @param mode 不为0则进ISP模式
void STCReset(ResetMode mode)
{
    SWBS = (mode ? 1 : 0);
    SWRST = 1;
}

// @brief 擦除E2PROM
// @param IAPAddr 擦除在这个地址上的扇区(一个扇区512B)
void IAPErase(unsigned short IAPAddr)
{
    IAPEN = 1;
    IAP_ADDRL = IAPAddr;
    IAP_ADDRH = IAPAddr >> 8;
    IAPExec(IAPCMD_Erase);
    IAPEN = 0;
}

// @brief 读E2PROM
// @param IAPAddr E2PROM上此地址
// @param addr 内存上的地址
// @param len 读写数据的长度
void IAPRead(unsigned short IAPAddr, DataAddr addr, DataLen len)
{
    for (; len > 0; len--, addr++, IAPAddr++)
    {
        *addr = *(unsigned char far *)(0xfe0000 + IAPAddr);
    }
}

// @brief 写E2PROM
// @param IAPAddr E2PROM上此地址
// @param addr 内存上的地址
// @param len 读写数据的长度
void IAPWrite(unsigned short IAPAddr, DataAddr addr, DataLen len)
{
    IAPEN = 1;
    IAP_ADDRE = 0;
    IAP_ADDRL = IAPAddr;
    IAP_ADDRH = IAPAddr >> 8;
    for (; len > 0; len--, addr++)
    {
        IAP_DATA = *addr;
        IAPExec(IAPCMD_Write);
        if (IAP_ADDRL == 0xff)
        {
            IAP_ADDRH++;
            IAP_ADDRL = 0;
        }
        else
        {
            IAP_ADDRL++;
        }
    }
    IAPEN = 0;
}

// @brief 初始化外部中断
// @param priority2b 中断优先级 0-3 对INT2 INT3无效
// @param mode 外部中断模式 非0只检测升降 对INT0 INT1生效 其它只检测下降
// @param iNum 外部中断编号 0-4
void INTInit(Priority priority2b, INTMode mode, INTNum iNum)
{
    switch (iNum)
    {
    case 0:
        if (mode & 0x04)
        {
            EX0 = 0;
        }
        else
        {
            EX0 = 1;
            IT0 = mode ? 0 : 1;
            PX0 = priority2b & 0x01;
            PX0H = priority2b & 0x02;
        }
        break;
    case 1:
        if (mode & 0x04)
        {
            EX1 = 0;
        }
        else
        {
            EX1 = 1;
            IT1 = mode ? 0 : 1;
            PX1 = priority2b & 0x01;
            PX1H = priority2b & 0x02;
        }
        break;
    case 2:
        if (mode & 0x04)
        {
            EX2 = 0;
        }
        else
        {
            EX2 = 1;
        }

        break;
    case 3:
        if (mode & 0x04)
        {
            EX3 = 0;
        }
        else
        {
            EX3 = 1;
        }

        break;
    case 4:
        if (mode & 0x04)
        {
            EX4 = 0;
        }
        else
        {
            EX4 = 1;
            PX4 = priority2b & 0x01;
            PX4H = priority2b & 0x02;
        }
        break;
    }
}

// @brief 定时器/计数器初始化
// @param priority2b 中断优先级 0-3 对TMR0 TMR1生效
// @param mode 定时器模式
// @param tNum 定时器编号 0-4
// @param freq 定时器工作频率
void TMRInit(Priority priority2b, TMRMode mode, TMRNum tNum, float freq)
{
    switch (tNum)
    {
    case 0:
        TR0 = 0;
        T0_M1 = 0;
        if (mode & 0x02)
        {
            unsigned long t;
            T0_M0 = 0;
            T0_CT = 0;
            ET0 = 1;
            t = (unsigned long)((SYSclk / freq) + 0.5);
            if (t & 0xf000)
            {
                T0x12 = 0;
                t /= 12;
            }
            else
            {
                T0x12 = 1;
            }
            TM0PS = (unsigned char)(t >> 16);
            t /= TM0PS + 1;
            t &= 0x0000ffff;
            t = 0x00010000 - t;
            TH0 = (unsigned char)(t >> 8);
            TL0 = (unsigned char)(t);
            PT0 = priority2b & 0x01;
            PT0H = priority2b & 0x02;
            TR0 = 1;
        }
        else
        {
            if (mode & 0x01)
            {
                T0_CT = 1;
                T0_M0 = 1;
                ET0 = 0;
                TH0 = 0;
                TL0 = 0;
                TR0 = 1;
            }
        }
        T0CLKO = mode & 0x80;
        break;
    case 1:
        TR1 = 0;
        T1_M1 = 0;
        if (mode & 0x02)
        {
            unsigned long t;
            T1_CT = 0;
            T1_M0 = 0;
            if (mode & 0x01)
            {
                freq *= 4;
                ET1 = 0;
            }
            else
            {
                ET1 = 1;
            }
            t = (unsigned long)((SYSclk / freq) + 0.5);
            if (t & 0xf000)
            {
                T1x12 = 0;
                t /= 12;
            }
            else
            {
                T1x12 = 1;
            }
            TM1PS = (unsigned char)(t >> 16);
            t /= TM1PS + 1;
            t &= 0x0000ffff;
            t = 0x00010000 - t;
            TH1 = (unsigned char)(t >> 8);
            TL1 = (unsigned char)(t);
            PT1 = priority2b & 0x01;
            PT1H = priority2b & 0x02;
            TR1 = 1;
        }
        else
        {
            if (mode & 0x01)
            {
                T1_CT = 1;
                T1_M0 = 1;
                ET1 = 0;
                TH1 = 0;
                TL1 = 0;
                TR1 = 1;
            }
        }
        T1CLKO = mode & 0x80;
        break;
    case 2:
        T2R = 0;
        if (mode & 0x02)
        {
            unsigned long t;
            T2_CT = 0;
            ET2 = 1;
            if (mode & 0x01)
            {
                freq *= 4;
                ET2 = 0;
            }
            else
            {
                ET2 = 1;
            }
            t = (unsigned long)((SYSclk / freq) + 0.5);
            if (t & 0xf000)
            {
                T2x12 = 0;
                t /= 12;
            }
            else
            {
                T2x12 = 1;
            }
            TM2PS = (unsigned char)(t >> 16);
            t /= TM2PS + 1;
            t &= 0x0000ffff;
            t = 0x00010000 - t;
            T2H = (unsigned char)(t >> 8);
            T2L = (unsigned char)(t);
            T2R = 1;
        }
        else
        {
            if (mode & 0x01)
            {
                T2_CT = 1;
                ET2 = 0;
                T2H = 0;
                T2L = 0;
                T2R = 1;
            }
        }
        T2CLKO = mode & 0x80;
        break;
    case 3:
        T3R = 0;
        if (mode & 0x02)
        {
            unsigned long t;
            T3_CT = 0;
            ET3 = 1;
            if (mode & 0x01)
            {
                freq *= 4;
                ET3 = 0;
            }
            else
            {
                ET3 = 1;
            }
            t = (unsigned long)((SYSclk / freq) + 0.5);
            if (t & 0xf000)
            {
                T3x12 = 0;
                t /= 12;
            }
            else
            {
                T3x12 = 1;
            }
            TM3PS = (unsigned char)(t >> 16);
            t /= TM3PS + 1;
            t &= 0x0000ffff;
            t = 0x00010000 - t;
            T3H = (unsigned char)(t >> 8);
            T3L = (unsigned char)(t);
            T3R = 1;
        }
        else
        {
            if (mode & 0x01)
            {
                T3_CT = 1;
                ET3 = 0;
                T3H = 0;
                T3L = 0;
                T3R = 1;
            }
        }
        T3CLKO = mode & 0x80;
        break;
    case 4:
        T4R = 0;
        if (mode & 0x02)
        {
            unsigned long t;
            T4_CT = 0;
            ET4 = 1;
            if (mode & 0x01)
            {
                freq *= 4;
                ET4 = 0;
            }
            else
            {
                ET4 = 1;
            }
            t = (unsigned long)((SYSclk / freq) + 0.5);
            if (t & 0xf000)
            {
                T4x12 = 0;
                t /= 12;
            }
            else
            {
                T4x12 = 1;
            }
            TM4PS = (unsigned char)(t >> 16);
            t /= TM4PS + 1;
            t &= 0x0000ffff;
            t = 0x00010000 - t;
            T4H = (unsigned char)(t >> 8);
            T4L = (unsigned char)(t);
            T4R = 1;
        }
        else
        {
            if (mode & 0x01)
            {
                T4_CT = 1;
                ET4 = 0;
                T4H = 0;
                T4L = 0;
                T4R = 1;
            }
        }
        T4CLKO = mode & 0x80;
        break;
    }
}

// @brief 读定时器值并清零
// @param tNum 定时器编号 0-4
// @param dataAddr 保存数据地址(unsigned short)
void TMRRead(TMRNum tNum, DataAddr dataAddr)
{
    switch (tNum)
    {
    case 0:
        *dataAddr = TH0;
        *(dataAddr + 1) = TL0;
        TL0 = 0;
        TH0 = 0;
        break;
    case 1:
        *dataAddr = TH1;
        *(dataAddr + 1) = TL1;
        TL1 = 0;
        TH1 = 0;
        break;
    case 2:
        *dataAddr = T2H;
        *(dataAddr + 1) = T2L;
        T2R = 0;
        T2L = 0;
        T2H = 0;
        T2R = 1;
        break;
    case 3:
        *dataAddr = T3H;
        *(dataAddr + 1) = T3L;
        T3R = 0;
        T3L = 0;
        T3H = 0;
        T3R = 1;
        break;
    case 4:
        *dataAddr = T4H;
        *(dataAddr + 1) = T4L;
        T4R = 0;
        T4L = 0;
        T4H = 0;
        T4R = 1;
        break;
    }
}

void TMR0_isr(void) interrupt TMR0_VECTOR using 1
{
    ULibTMRTaskPlus(&TMR0, 1);
    ULibTMRTaskCheck(&TMR0);
}
void TMR1_isr(void) interrupt TMR1_VECTOR using 1
{
    ULibTMRTaskPlus(&TMR1, 1);
    ULibTMRTaskCheck(&TMR1);
}
void TMR2_isr(void) interrupt TMR2_VECTOR using 1
{
    ULibTMRTaskPlus(&TMR2, 1);
    ULibTMRTaskCheck(&TMR2);
}
void TMR3_isr(void) interrupt TMR3_VECTOR using 1
{
    ULibTMRTaskPlus(&TMR3, 1);
    ULibTMRTaskCheck(&TMR3);
}
void TMR4_isr(void) interrupt TMR4_VECTOR using 1
{
    ULibTMRTaskPlus(&TMR4, 1);
    ULibTMRTaskCheck(&TMR4);
}

// @brief 初始化PWM主时钟
// @param group PWM组 A->0 B->1
// @param group PWM频率
void PWMClkInit(PWMGroup group, float freq)
{
    switch (group)
    {
    case 0:
    {
        unsigned long t0;
        unsigned short t1, t2;
        PWMA_SMCR &= 0xf8;
        t0 = SYSclk / ((PWMA_CR1 & 0x60) ? freq * 2 : freq);
        t1 = t0 >> 16;
        t2 = (t0 / (t1 + 1)) - 1;
        PWMA_CR1 &= 0x7f;
        PWMA_PSCRH = t1 >> 8;
        PWMA_PSCRL = t1;
        PWMA_ARRH = t2 >> 8;
        PWMA_ARRL = t2;
        PWMA_CR1 |= 0x81;
        break;
    }
    case 1:
    {
        unsigned long t0;
        unsigned short t1, t2;
        PWMB_SMCR &= 0xf8;
        t0 = SYSclk / ((PWMB_CR1 & 0x60) ? freq * 2 : freq);
        t1 = t0 >> 16;
        t2 = (t0 / (t1 + 1)) - 1;
        PWMB_CR1 &= 0x7f;
        PWMB_PSCRH = t1 >> 8;
        PWMB_PSCRL = t1;
        PWMB_ARRH = t2 >> 8;
        PWMB_ARRL = t2;
        PWMB_CR1 |= 0x81;
        break;
    }
    }
}

// @brief 初始化内存复制
// @param priority4b b2-b3中断优先级 b0-b1总线优先级
// @param mode 内存复制模式
void M2MInit(Priority priority4b, M2MMode mode)
{
    DMA_M2M_CFG = 0x80;
    DMA_M2M_CFG |= mode << 4;
    DMA_M2M_CFG |= priority4b & 0x0f;
}

// @brief 内存复制
// @param taskAddr 任务数据地址
// @param nextTaskBreakPoint 下一断点 如果失败将会停留在当前断点
// @param srcAddr 源内存地址
// @param distAddr 目标内存地址
// @param len 复制数据长度
void M2MCopy(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, DataAddr srcAddr, DataAddr distAddr, DataLen len)
{
    if (M2MTaskAddr == AddrNull)
    {
        if (taskAddr != AddrNull)
        {
            taskAddr->bp = nextTaskBreakPoint;
        }
        if (len)
        {
            len--;
            if (taskAddr != AddrNull)
            {
                M2MTaskAddr = taskAddr;
                taskAddr->s = ULibTaskState_Idle;
            }
            else
            {
                M2MTaskAddr = AddrNoop;
            }
            DMA_M2M_TXAL = (unsigned char)(unsigned short)srcAddr;
            DMA_M2M_TXAH = (unsigned char)((unsigned short)srcAddr >> 8);
            DMA_M2M_RXAL = (unsigned char)(unsigned short)distAddr;
            DMA_M2M_RXAH = (unsigned char)((unsigned short)distAddr >> 8);
            DMA_M2M_AMT = len;
            DMA_M2M_AMTH = len >> 8;
            DMA_M2M_CR = 0xc0;
        }
    }
}

void M2M_isr(void) interrupt DMA_M2M_VECTOR using 2
{
    if (M2MTaskAddr != AddrNull && M2MTaskAddr != AddrNoop)
    {
        M2MTaskAddr->s = ULibTaskState_Active;
    }
    M2MTaskAddr = AddrNull;
    DMA_M2M_STA = 0x00;
}

// @brief初始化ADC
// @param priority4b b2-b3中断优先级 b0-b1总线优先级
// @param mode ADC模式(采样次数)
void ADCInit(Priority priority4b, ADCMode mode)
{
    ADCTIM = 0x3f;
    ADCCFG |= mode & 0x20;
    ADC_CONTR = 0x80;
    DMA_ADC_CFG = 0x80;
    DMA_ADC_CFG2 = mode & 0x0f;
    DMA_ADC_CFG |= priority4b & 0x0f;
}

// @brief ADC采样值并赋值到数据地址 425.532k@40M
// @param taskAddr 任务数据地址
// @param nextTaskBreakPoint 下一断点 如果失败将会停留在当前断点
// @param pins 开启的通道
// @param dataAddr 保存数据xdata地址
void ADCRead(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, ADCPins pins, DataAddr dataAddr)
{
    if (ADCTaskAddr == AddrNull)
    {
        if (taskAddr != AddrNull)
        {
            ADCTaskAddr = taskAddr;
            taskAddr->s = ULibTaskState_Idle;
            taskAddr->bp = nextTaskBreakPoint;
        }
        else
        {
            ADCTaskAddr = AddrNoop;
        }
        DMA_ADC_RXAL = (unsigned char)(unsigned short)dataAddr;
        DMA_ADC_RXAH = (unsigned char)((unsigned short)dataAddr >> 8);
        DMA_ADC_CHSW1 = (unsigned char)(pins >> 8);
        DMA_ADC_CHSW0 = (unsigned char)pins;
        DMA_ADC_CR = 0xc0;
    }
}

void ADC_isr(void) interrupt DMA_ADC_VECTOR using 2
{
    if (ADCTaskAddr != AddrNull && ADCTaskAddr != AddrNoop)
    {
        ADCTaskAddr->s = ULibTaskState_Active;
    }
    ADCTaskAddr = AddrNull;
    DMA_ADC_STA = 0x00;
}

// @brief 设置SPI引脚
// @param pin SPI引脚
void SPIPin(SPIPins pins)
{
    P_SW1 &= 0xf3;
    P_SW1 |= pins & 0x0c;
    DMA_SPI_CFG2 = 0x04;
    DMA_SPI_CFG2 |= pins & 0x03;
}

// @brief 初始化SPI
// @param priority4b b2-b3中断优先级 b0-b1总线优先级
// @param mode SPI模式
void SPIInit(Priority priority4b, SPIMode mode)
{
    SPCTL = 0xc0;
    SPSTAT = 0xc0;
    HSSPI_CFG2 |= 0x30;
    DMA_SPI_CFG = 0x80;
    SPCTL |= mode & 0x3f;
    DMA_SPI_CFG |= priority4b & 0x0f;
}

// @brief SPI收发
// @param taskAddr 任务数据地址
// @param nextTaskBreakPoint 下一断点 如果失败将会停留在当前断点
// @param txAddr 发送数据xdata地址
// @param rxAddr 接收数据xdata地址
// @param len 收发数据长度 若为0直接跳到下一个断点
void SPIShift(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, DataAddr txAddr, DataAddr rxAddr, DataLen len)
{
    if (SPITaskAddr == AddrNull)
    {
        if (taskAddr != AddrNull)
        {
            taskAddr->bp = nextTaskBreakPoint;
        }
        if (len)
        {
            len--;
            if (taskAddr != AddrNull)
            {
                SPITaskAddr = taskAddr;
                taskAddr->s = ULibTaskState_Idle;
            }
            else
            {
                SPITaskAddr = AddrNoop;
            }
            if (txAddr != AddrNull)
            {
                DMA_SPI_TXAL = (unsigned char)(unsigned short)txAddr;
                DMA_SPI_TXAH = (unsigned char)((unsigned short)txAddr >> 8);
                DMA_SPI_CFG |= 0x40;
            }
            else
            {
                DMA_SPI_CFG |= ~0x40;
            }
            if (rxAddr != AddrNull)
            {
                DMA_SPI_RXAL = (unsigned char)(unsigned short)rxAddr;
                DMA_SPI_RXAH = (unsigned char)((unsigned short)rxAddr >> 8);
                DMA_SPI_CFG |= 0x20;
            }
            else
            {
                DMA_SPI_CFG |= ~0x20;
            }
            DMA_SPI_AMT = len;
            DMA_SPI_AMTH = len >> 8;
            if (SPCTL & 0x10)
            {
                DMA_SPI_CR = 0xc1;
            }
            else
            {
                DMA_SPI_CR = 0xa1;
            }
        }
    }
}

void SPI_isr(void) interrupt DMA_SPI_VECTOR using 2
{
    if (SPITaskAddr != AddrNull && SPITaskAddr != AddrNoop)
    {
        SPITaskAddr->s = ULibTaskState_Active;
        SPITaskAddr = AddrNull;
    }
    DMA_SPI_STA = 0x00;
}

void I2CPin(I2CPins pin)
{
    P_SW2 &= 0xcf;
    P_SW2 |= (pin & 0x03) << 4;
}

// @brief 初始化I2C 主机模式从机模式懒得写了
// @param priority8b b7-b6接收中断优先级 b5-b4接收总线优先级 b3-b2发送中断优先级 b1-b0发送总线优先级
// @param freq I2C频率 不知道为什么主时钟40M时下不去300k
void I2CInit(Priority priority8b, unsigned long freq)
{
    unsigned long t;
    I2CCFG = 0xc0;
    I2CMSST = 0x00;
    t = (SYSclk / (freq + 2));
    t = t > (27 << 2) ? 27 : t >> 2;
    I2CCFG |= t;
    DMA_I2CT_CFG = 0x80;
    DMA_I2CT_CFG |= priority8b & 0x0f;
    DMA_I2CT_CR = 0x80;
    DMA_I2CR_CFG = 0x80;
    DMA_I2CR_CFG |= (priority8b & 0xf0) >> 4;
    DMA_I2CR_CR = 0x81;
}

// @brief I2C发送
// @param taskAddr 任务数据地址
// @param nextTaskBreakPoint 下一断点 如果失败将会停留在当前断点
// @param txAddr 发送数据xdata地址
// @param txlen 发送数据长度
void I2CSend(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, DataAddr txAddr, DataLen txlen)
{
    if (I2CTTaskAddr == AddrNull && I2CRTaskAddr == AddrNull)
    {
        if (taskAddr != AddrNull)
        {
            taskAddr->bp = nextTaskBreakPoint;
        }
        if (txlen)
        {
            DMA_I2C_CR = 0x00;
            if (taskAddr != AddrNull)
            {
                I2CTTaskAddr = taskAddr;
                taskAddr->s = ULibTaskState_Idle;
            }
            else
            {
                I2CTTaskAddr = AddrNoop;
            }
            *txAddr &= 0xfe;
            txlen--;
            DMA_I2C_ST1 = txlen;
            DMA_I2C_ST2 = txlen >> 8;
            DMA_I2CT_AMT = DMA_I2C_ST1;
            DMA_I2CT_AMTH = DMA_I2C_ST2;
            DMA_I2CT_TXAL = (unsigned char)(unsigned short)txAddr;
            DMA_I2CT_TXAH = (unsigned char)((unsigned short)txAddr >> 8);
            I2CMSCR = 0x09;
            DMA_I2C_CR = 0x01;
            DMA_I2CT_CR |= 0x40;
        }
    }
}

// @brief I2C接收
// @param taskAddr 任务数据地址
// @param nextTaskBreakPoint 下一断点 如果失败将会停留在当前断点
// @param txAddr 发送数据xdata地址
// @param rxAddr 接收数据xdata地址 可为AddrNull表示只发送
// @param rxlen 接收数据长度 可为0表示只发送
void I2CRecv(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, DataAddr txAddr, DataAddr rxAddr, DataLen rxlen)
{
    if (I2CRTaskAddr == AddrNull && I2CTTaskAddr == AddrNull)
    {
        if (taskAddr != AddrNull)
        {
            taskAddr->bp = nextTaskBreakPoint;
        }
        if (rxlen)
        {
            DMA_I2C_CR = 0x00;
            if (taskAddr != AddrNull)
            {
                I2CRTaskAddr = taskAddr;
                taskAddr->s = ULibTaskState_Idle;
            }
            else
            {
                I2CRTaskAddr = AddrNoop;
            }
            *txAddr |= 0x01;
            DMA_I2C_ST1 = rxlen;
            DMA_I2C_ST2 = rxlen >> 8;
            rxlen--;
            DMA_I2CT_AMT = 0;
            DMA_I2CT_AMTH = 0;
            DMA_I2CR_AMT = rxlen;
            DMA_I2CR_AMTH = rxlen >> 8;
            DMA_I2CT_TXAL = (unsigned char)(unsigned short)txAddr;
            DMA_I2CT_TXAH = (unsigned char)((unsigned short)txAddr >> 8);
            DMA_I2CR_RXAL = (unsigned char)(unsigned short)rxAddr;
            DMA_I2CR_RXAH = (unsigned char)((unsigned short)rxAddr >> 8);
            I2CMSCR = 0x09;
            DMA_I2C_CR = 0x01;
            DMA_I2CT_CR |= 0x40;
            DMA_I2CR_CR |= 0x41;
        }
    }
}

void I2CT_isr(void) interrupt DMA_I2CT_VECTOR using 2
{
    if (I2CTTaskAddr != AddrNull && I2CTTaskAddr != AddrNoop)
    {
        I2CTTaskAddr->s = ULibTaskState_Active;
    }
    else if (I2CRTaskAddr != AddrNull)
    {
        I2CMSCR = 0x0b;
    }
    I2CTTaskAddr = AddrNull;
    DMA_I2CT_STA = 0x00;
}
void I2CR_isr(void) interrupt DMA_I2CR_VECTOR using 2
{
    if (I2CRTaskAddr != AddrNull && I2CRTaskAddr != AddrNoop)
    {
        I2CRTaskAddr->s = ULibTaskState_Active;
    }
    I2CRTaskAddr = AddrNull;
    DMA_I2CR_STA = 0x00;
}

// @brief 串口引脚配置
// @param uNum 串口编号
// @param pin 串口引脚
void URTPin(URTNum uNum, URTPins pin)
{
    switch (uNum)
    {
    case 1:
        P_SW1 &= 0x3f;
        P_SW1 |= (pin & 0x03) << 6;
        break;
    case 2:
        P_SW2 &= 0xfe;
        P_SW2 |= pin ? 0x01 : 0x00;
        break;
    case 3:
        P_SW2 &= 0xfd;
        P_SW2 |= pin ? 0x02 : 0x00;
        break;
    case 4:
        P_SW2 &= 0xfb;
        P_SW2 |= pin ? 0x04 : 0x00;
        break;
    }
}

// @brief 串口初始化
// @param priority8b b7-b6接收中断优先级 b5-b4接收总线优先级 b3-b2发送中断优先级 b1-b0发送总线优先级
// @param mode 串口模式
// @param uNum 串口编号
// @param tNum 波特率定时器编号
void URTInit(Priority priority8b, URTMode mode, URTNum uNum, TMRNum tNum)
{
    switch (uNum)
    {
    case 1:
        SM0 = 0;
        SM1 = 1;
        SM2 = 0;
        USARTCR1 = 0x00;
        REN = !(mode & 0x01);
        if (tNum == 1)
        {
            S1BRT = 0;
        }
        else
        {
            S1BRT = 1;
        }
        DMA_UR1T_CFG = 0x80;
        DMA_UR1T_CFG |= priority8b & 0x0f;
        DMA_UR1R_CFG = 0x80;
        DMA_UR1R_CFG |= (priority8b & 0xf0) >> 4;

        break;
    case 2:
        S2SM0 = 0;
        S2SM1 = 1;
        S2SM2 = 0;
        S2CFG |= 0x01;
        S2REN = !(mode & 0x01);
        DMA_UR2T_CFG = 0x80;
        DMA_UR2T_CFG |= priority8b & 0x0f;
        DMA_UR2R_CFG = 0x80;
        DMA_UR2R_CFG |= (priority8b & 0xf0) >> 4;

        break;
    case 3:
        S3SM0 = 0;
        S3SM2 = 0;
        S3REN = !(mode & 0x01);
        if (tNum == 3)
        {
            S3ST3 = 1;
        }
        else
        {
            S3ST3 = 0;
        }
        DMA_UR3T_CFG = 0x80;
        DMA_UR3T_CFG |= priority8b & 0x0f;
        DMA_UR3R_CFG = 0x80;
        DMA_UR3R_CFG |= (priority8b & 0xf0) >> 4;
        break;
    case 4:
        S4SM0 = 0;
        S4SM2 = 0;
        S4REN = !(mode & 0x01);
        if (tNum == 4)
        {
            S4ST4 = 1;
        }
        else
        {
            S4ST4 = 0;
        }
        DMA_UR4T_CFG = 0x80;
        DMA_UR4T_CFG |= priority8b & 0x0f;
        DMA_UR4R_CFG = 0x80;
        DMA_UR4R_CFG |= (priority8b & 0xf0) >> 4;
        break;
    }
}

// @brief 串口发送
// @param taskAddr 任务数据地址
// @param nextTaskBreakPoint 下一断点 如果失败将会停留在当前断点
// @param uNum 串口编号 1-4
// @param dataAddr 发送数据xdata地址
// @param len 发送数据长度
void URTSend(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, URTNum uNum, DataAddr dataAddr, DataLen len)
{
    switch (uNum)
    {
    case 1:
        if (UR1TTaskAddr == AddrNull)
        {
            if (taskAddr != AddrNull)
            {
                taskAddr->bp = nextTaskBreakPoint;
            }
            if (len)
            {
                len--;
                if (taskAddr != AddrNull)
                {
                    UR1TTaskAddr = taskAddr;
                    taskAddr->s = ULibTaskState_Idle;
                }
                else
                {
                    UR1TTaskAddr = AddrNoop;
                }
                DMA_UR1T_TXAL = (unsigned char)(unsigned short)dataAddr;
                DMA_UR1T_TXAH = (unsigned char)((unsigned short)dataAddr >> 8);
                DMA_UR1T_AMT = len;
                DMA_UR1T_AMTH = len >> 8;
                DMA_UR1T_CR = 0xc0;
            }
        }
        break;
    case 2:
        if (UR2TTaskAddr == AddrNull)
        {
            if (taskAddr != AddrNull)
            {
                taskAddr->bp = nextTaskBreakPoint;
            }
            if (len)
            {
                len--;
                if (taskAddr != AddrNull)
                {
                    UR2TTaskAddr = taskAddr;
                    taskAddr->s = ULibTaskState_Idle;
                }
                else
                {
                    UR2TTaskAddr = AddrNoop;
                }
                DMA_UR2T_TXAL = (unsigned char)(unsigned short)dataAddr;
                DMA_UR2T_TXAH = (unsigned char)((unsigned short)dataAddr >> 8);
                DMA_UR2T_AMT = len;
                DMA_UR2T_AMTH = len >> 8;
                DMA_UR2T_CR = 0xc0;
            }
        }
        break;
    case 3:
        if (UR3TTaskAddr == AddrNull)
        {
            if (taskAddr != AddrNull)
            {
                taskAddr->bp = nextTaskBreakPoint;
            }
            if (len)
            {
                len--;
                if (taskAddr != AddrNull)
                {
                    UR3TTaskAddr = taskAddr;
                    taskAddr->s = ULibTaskState_Idle;
                }
                else
                {
                    UR3TTaskAddr = AddrNoop;
                }
                DMA_UR3T_TXAL = (unsigned char)(unsigned short)dataAddr;
                DMA_UR3T_TXAH = (unsigned char)((unsigned short)dataAddr >> 8);
                DMA_UR3T_AMT = len;
                DMA_UR3T_AMTH = len >> 8;
                DMA_UR3T_CR = 0xc0;
            }
        }
        break;
    case 4:
        if (UR4TTaskAddr == AddrNull)
        {
            if (taskAddr != AddrNull)
            {
                taskAddr->bp = nextTaskBreakPoint;
            }
            if (len)
            {
                len--;
                if (taskAddr != AddrNull)
                {
                    UR4TTaskAddr = taskAddr;
                    taskAddr->s = ULibTaskState_Idle;
                }
                else
                {
                    UR4TTaskAddr = AddrNoop;
                }
                DMA_UR4T_TXAL = (unsigned char)(unsigned short)dataAddr;
                DMA_UR4T_TXAH = (unsigned char)((unsigned short)dataAddr >> 8);
                DMA_UR4T_AMT = len;
                DMA_UR4T_AMTH = len >> 8;
                DMA_UR4T_CR = 0xc0;
            }
        }
        break;
    }
}

// @brief 串口接收
// @param taskAddr 任务数据地址
// @param nextTaskBreakPoint 下一断点 如果失败将会停留在当前断点
// @param uNum 串口编号 1-4
// @param dataAddr 接收数据xdata地址
// @param len 接收数据长度
void URTRecv(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, URTNum uNum, DataAddr dataAddr, DataLen len)
{

    switch (uNum)
    {
    case 1:
        if (UR1RTaskAddr == AddrNull)
        {
            if (taskAddr != AddrNull)
            {
                taskAddr->bp = nextTaskBreakPoint;
            }
            if (len)
            {
                len--;
                if (taskAddr != AddrNull)
                {
                    UR1RTaskAddr = taskAddr;
                    taskAddr->s = ULibTaskState_Idle;
                }
                else
                {
                    UR1RTaskAddr = AddrNoop;
                }
                DMA_UR1R_RXAL = (unsigned char)(unsigned short)dataAddr;
                DMA_UR1R_RXAH = (unsigned char)((unsigned short)dataAddr >> 8);
                DMA_UR1R_AMT = len;
                DMA_UR1R_AMTH = len >> 8;
                DMA_UR1R_CR = 0xa1;
            }
        }
        break;
    case 2:
        if (UR2RTaskAddr == AddrNull)
        {
            if (taskAddr != AddrNull)
            {
                taskAddr->bp = nextTaskBreakPoint;
            }
            if (len)
            {
                len--;
                if (taskAddr != AddrNull)
                {
                    UR2RTaskAddr = taskAddr;
                    taskAddr->s = ULibTaskState_Idle;
                }
                else
                {
                    UR2RTaskAddr = AddrNoop;
                }
                DMA_UR2R_RXAL = (unsigned char)(unsigned short)dataAddr;
                DMA_UR2R_RXAH = (unsigned char)((unsigned short)dataAddr >> 8);
                DMA_UR2R_AMT = len;
                DMA_UR2R_AMTH = len >> 8;
                DMA_UR2R_CR = 0xa1;
            }
        }
        break;
    case 3:
        if (UR3RTaskAddr == AddrNull)
        {
            if (taskAddr != AddrNull)
            {
                taskAddr->bp = nextTaskBreakPoint;
            }
            if (len)
            {
                len--;
                if (taskAddr != AddrNull)
                {
                    UR3RTaskAddr = taskAddr;
                    taskAddr->s = ULibTaskState_Idle;
                }
                else
                {
                    UR3RTaskAddr = AddrNoop;
                }
                DMA_UR3R_RXAL = (unsigned char)(unsigned short)dataAddr;
                DMA_UR3R_RXAH = (unsigned char)((unsigned short)dataAddr >> 8);
                DMA_UR3R_AMT = len;
                DMA_UR3R_AMTH = len >> 8;
                DMA_UR3R_CR = 0xa1;
            }
        }
        break;
    case 4:
        if (UR4RTaskAddr == AddrNull)
        {
            if (taskAddr != AddrNull)
            {
                taskAddr->bp = nextTaskBreakPoint;
            }
            if (len)
            {
                len--;
                if (taskAddr != AddrNull)
                {
                    UR4RTaskAddr = taskAddr;
                    taskAddr->s = ULibTaskState_Idle;
                }
                else
                {
                    UR4RTaskAddr = AddrNoop;
                }
                DMA_UR4R_RXAL = (unsigned char)(unsigned short)dataAddr;
                DMA_UR4R_RXAH = (unsigned char)((unsigned short)dataAddr >> 8);
                DMA_UR4R_AMT = len;
                DMA_UR4R_AMTH = len >> 8;
                DMA_UR4R_CR = 0xa1;
            }
        }
        break;
    }
}

void UR1T_isr(void) interrupt DMA_UR1T_VECTOR using 3
{
    if (UR1TTaskAddr != AddrNull && UR1TTaskAddr != AddrNoop)
    {
        UR1TTaskAddr->s = ULibTaskState_Active;
    }
    UR1TTaskAddr = AddrNull;
    DMA_UR1T_STA = 0x00;
}
void UR1R_isr(void) interrupt DMA_UR1R_VECTOR using 3
{
    if (UR1RTaskAddr != AddrNull && UR1RTaskAddr != AddrNoop)
    {
        UR1RTaskAddr->s = ULibTaskState_Active;
    }
    UR1RTaskAddr = AddrNull;
    DMA_UR1R_STA = 0x00;
}
void UR2T_isr(void) interrupt DMA_UR2T_VECTOR using 3
{
    if (UR2TTaskAddr != AddrNull && UR2TTaskAddr != AddrNoop)
    {
        UR2TTaskAddr->s = ULibTaskState_Active;
    }
    UR2TTaskAddr = AddrNull;
    DMA_UR2T_STA = 0x00;
}
void UR2R_isr(void) interrupt DMA_UR2R_VECTOR using 3
{
    if (UR2RTaskAddr != AddrNull && UR2RTaskAddr != AddrNoop)
    {
        UR2RTaskAddr->s = ULibTaskState_Active;
    }
    UR2RTaskAddr = AddrNull;
    DMA_UR2R_STA = 0x00;
}
void UR3T_isr(void) interrupt DMA_UR3T_VECTOR using 3
{
    if (UR3TTaskAddr != AddrNull && UR3TTaskAddr != AddrNoop)
    {
        UR3TTaskAddr->s = ULibTaskState_Active;
    }
    UR3TTaskAddr = AddrNull;
    DMA_UR3T_STA = 0x00;
}
void UR3R_isr(void) interrupt DMA_UR3R_VECTOR using 3
{
    if (UR3RTaskAddr != AddrNull && UR3RTaskAddr != AddrNoop)
    {
        UR3RTaskAddr->s = ULibTaskState_Active;
    }
    UR3RTaskAddr = AddrNull;
    DMA_UR3R_STA = 0x00;
}
void UR4T_isr(void) interrupt DMA_UR4T_VECTOR using 3
{
    if (UR4TTaskAddr != AddrNull && UR4TTaskAddr != AddrNoop)
    {
        UR4TTaskAddr->s = ULibTaskState_Active;
    }
    UR4TTaskAddr = AddrNull;
    DMA_UR4T_STA = 0x00;
}
void UR4R_isr(void) interrupt DMA_UR4R_VECTOR using 3
{
    if (UR4RTaskAddr != AddrNull && UR4RTaskAddr != AddrNoop)
    {
        UR4RTaskAddr->s = ULibTaskState_Active;
    }
    UR4RTaskAddr = AddrNull;
    DMA_UR4R_STA = 0x00;
}