#ifndef __ULibSTC32_H__
#define __ULibSTC32_H__
#include "STC32G.h"
#include "ULibTask.h"

#define AddrNull (void *)0
#define AddrNoop (void *)1

#define INTPin_0 P32
#define INTPin_1 P33
#define INTPin_2 P36
#define INTPin_3 P37
#define INTPin_4 P30

#define TMRPin_Count0 P34
#define TMRPin_ClkO0 P35
#define TMRPin_Count1 P35
#define TMRPin_ClkO1 P34
#define TMRPin_Count2 P54
#define TMRPin_ClkO2 P13
#define TMRPin_Count3 (T3T4PS ? P00 : P04)
#define TMRPin_ClkO3 (T3T4PS ? P01 : P05)
#define TMRPin_Count4 (T3T4PS ? P02 : P06)
#define TMRPin_ClkO4 (T3T4PS ? P03 : P07)

typedef unsigned char *DataAddr;
typedef unsigned char Priority; // b7-b6 RxInt b5-b4 RxBus b3-b2 TxInt b1-b0 TxBus
typedef unsigned short DataLen;
typedef unsigned char CLKMode;
typedef unsigned char ResetMode;
typedef unsigned char GIOMode;
typedef unsigned char GIOPort; // 0-7
typedef unsigned char GIOMask; // b0-b7->pin0-pin7
typedef unsigned char INTMode;
typedef unsigned char INTNum;
typedef unsigned char TMRMode;
typedef unsigned char TMRNum;   // all: 0, 1 ,2, 3, 4
typedef unsigned char PWMGroup; // all: 0, 1
typedef unsigned char M2MMode;
typedef unsigned char SPIMode;
typedef unsigned char SPIPins;
typedef unsigned char URTMode;
typedef unsigned char URTPins;
typedef unsigned char URTNum; // all: 0, 1 ,2, 3, 4; 0-2 SPI ;1-4 UART
typedef unsigned char ADCMode;
typedef unsigned short ADCPins; // P00-P07 P10-P16 1.19V
typedef unsigned char I2CPins;

#define CLKMode_22118400 0
#define CLKMode_24000000 1
#define CLKMode_27000000 2
#define CLKMode_30000000 3
#define CLKMode_33117600 4
#define CLKMode_35000000 5
#define CLKMode_36864000 6
#define CLKMode_40000000 7
#define CLKMode_44236800 8
#define CLKMode_48000000 9
#define CLKMode_Output1Div 10
#define CLKMode_Output2Div 20
#define CLKMode_Output4Div 30
#define CLKMode_Output8Div 40
#define CLKMode_Output16Div 50
#define CLKMode_Output32Div 60
#define CLKMode_Output64Div 70
#define CLKMode_OutputPin54 0
#define CLKMode_OutputPin16 100

#define ResetMode_Normal 0
#define ResetMode_Isp 1

#define GIOMode_IO 0x00
#define GIOMode_PP 0x01
#define GIOMode_HI 0x02
#define GIOMode_OD 0x03
#define GIOMode_PU4K1 0x04
#define GIOMode_PD10K 0x08
#define GIOMode_STiggerOff 0x10
#define GIOMode_D2CLKOff 0x20
#define GIOMode_DCurrentP 0x40
#define GIOMode_InputOff 0x80

#define IAPCMD_Noop 0
#define IAPCMD_Read 1
#define IAPCMD_Write 2
#define IAPCMD_Erase 3

#define INTMode_Down 0
#define INTMode_UpDown 1
#define INTMode_Off 4

#define TMRMode_Off 0x00
#define TMRMode_Counter 0x01
#define TMRMode_Timer 0x02
#define TMRMode_BaudGenerator 0x03 // TMR0没有此模式
#define TMRMode_CLKOutput 0x80

#define M2MMode_TpRp 0
#define M2MMode_TpRm 1
#define M2MMode_TmRp 2
#define M2MMode_TmRm 3

#define ADCMode_Cov1T 0x00
#define ADCMode_Cov2T 0x08
#define ADCMode_Cov4T 0x09
#define ADCMode_Cov8T 0x0a
#define ADCMode_Cov16T 0x0b
#define ADCMode_Cov32T 0x0c
#define ADCMode_Cov64T 0x0d
#define ADCMode_Cov128T 0x0e
#define ADCMode_Cov256T 0x0f
#define ADCMode_RESFMT 0x20

#define SPIMode_SPIDROP 0x20
#define SPIMode_SPIMSTR 0x10
#define SPIMode_SPICPOL 0x08
#define SPIMode_SPICPHA 0x04
#define SPIMode_SPICLK2Div 0x03
#define SPIMode_SPICLK16Div 0x02
#define SPIMode_SPICLK8Div 0x01
#define SPIMode_SPICLK4Div 0x00
#define SPIPin_SSS54 0x00
#define SPIPin_SSS22 0x01
#define SPIPin_SSS74 0x02
#define SPIPin_SSS35 0x03
#define SPIPin_SS54_MOSI13_MISO14_SCLK15 0x00
#define SPIPin_SS22_MOSI23_MISO24_SCLK25 0x04
#define SPIPin_SS54_MOSI40_MISO41_SCLK43 0x08
#define SPIPin_SS35_MOSI34_MISO33_SCLK32 0x0c
#define SPIPin_Slave 0x80

#define URTMode_8bit 0x00
#define URTMode_8bitNoRecv 0x01
#define URTPins_URT1_R30_T31 0
#define URTPins_URT1_R36_T37 1
#define URTPins_URT1_R16_T17 2
#define URTPins_URT1_R43_T44 3
#define URTPins_URT2_R10_T11 0
#define URTPins_URT2_R46_T47 1
#define URTPins_URT3_R00_T01 0
#define URTPins_URT3_R50_T51 1
#define URTPins_URT4_R02_T03 0
#define URTPins_URT4_R52_T53 1

#define I2CPins_SCL15_SDA14 0
#define I2CPins_SCL25_SDA24 1
#define I2CPins_SCL32_SDA33 3

#define BitSetLow(reg, mask) reg &= ~mask
#define BitSetHigh(reg, mask) reg |= mask
#define BitSet(reg, mask, hl)  \
    if (hl)                    \
    {                          \
        BitSetHigh(reg, mask); \
    }                          \
    else                       \
    {                          \
        BitSetLow(reg, mask);  \
    }
#define IAPExec(cmd) \
    IAP_CMD = cmd;   \
    IAP_TRIG = 0x5a; \
    IAP_TRIG = 0xa5
#define SPIMaster(sm)   \
    if (sm)             \
    {                   \
        SPCTL |= 0x10;  \
    }                   \
    else                \
    {                   \
        SPCTL &= ~0x10; \
    }

void STC32Init(void);
extern unsigned long SYSclk;
void CLKInit(CLKMode mode);
void GIOInit(GIOMode mode, GIOPort port, GIOMask mask);
void STCReset(ResetMode mode);

void IAPErase(unsigned short IAPAddr);
void IAPRead(unsigned short IAPAddr, DataAddr addr, DataLen len);
void IAPWrite(unsigned short IAPAddr, DataAddr addr, DataLen len);

void INTInit(Priority priority2b, INTMode mode, INTNum iNum);

extern ULibTMRTaskList TMR0, TMR1, TMR2, TMR3, TMR4;
void TMRInit(Priority priority2b, TMRMode mode, TMRNum tNum, float freq);
void TMRRead(TMRNum tNum, DataAddr dataAddr);

void PWMClkInit(PWMGroup group, float freq);

void M2MInit(Priority priority4b, M2MMode mode);
void M2MCopy(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, DataAddr srcAddr, DataAddr distAddr, DataLen len);

void ADCInit(Priority priority4b, ADCMode mode);
void ADCRead(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, ADCPins pins, DataAddr dataAddr);

void SPIPin(SPIPins pin);
void SPIInit(Priority priority4b, SPIMode mode);
void SPIShift(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, DataAddr txAddr, DataAddr rxAddr, DataLen len);

extern ULibTaskAddr I2CTTaskAddr;
extern ULibTaskAddr I2CRTaskAddr;
void I2CPin(I2CPins pin);
void I2CInit(Priority priority8b, unsigned long freq);
void I2CSend(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, DataAddr txAddr, DataLen txlen);
void I2CRecv(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, DataAddr txAddr, DataAddr rxAddr, DataLen rxlen);

extern ULibTaskAddr UR1RTaskAddr;
extern ULibTaskAddr UR2RTaskAddr;
extern ULibTaskAddr UR3RTaskAddr;
extern ULibTaskAddr UR4sRTaskAddr;
void URTPin(URTNum uNum, URTPins pin);
void URTInit(Priority priority8b, URTMode mode, URTNum uNum, TMRNum tNum);
void URTSend(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, URTNum uNum, DataAddr dataAddr, DataLen len);
void URTRecv(ULibTaskAddr taskAddr, ULibTaskBreakPoint nextTaskBreakPoint, URTNum uNum, DataAddr dataAddr, DataLen len);
#endif