#include "ULibTask.h"

// @brief 把任务放入任务列表
// @param taskList 任务列表
// @param taskAddr 任务地址
ULibTaskReturnCode ULibTaskPut(ULibTaskList *taskList, ULibTaskAddr taskAddr)
{
    short i;
    if (taskList->c >= ULibTaskList_MaxCount)
    {
        return ULibTaskReturnCode_Full;
    }
    for (i = taskList->c - 1; i >= 0 && taskList->tp[i]->p < taskAddr->p; i--)
    {
        taskList->tp[i + 1] = taskList->tp[i];
    }
    taskList->tp[i + 1] = taskAddr;
    taskList->c++;
    return ULibTaskReturnCode_Done;
}

// @brief 把任务弹出任务列表
// @param taskList 任务列表
// @param taskAddr 任务地址
ULibTaskReturnCode ULibTaskPop(ULibTaskList *taskList, ULibTaskAddr taskAddr)
{
    unsigned char i, j;
    if (taskList->c == 0)
    {
        return ULibTaskReturnCode_Empty;
    }
    for (i = 0, j = 0; i < taskList->c; i++)
    {
        if (j)
        {
            taskList->tp[i] = taskList->tp[i + 1];
        }
        else if (taskList->tp[i] == taskAddr)
        {
            taskList->tp[i] = taskList->tp[i + 1];
            taskList->c--;
            j = 1;
        }
    }
    if (!j)
    {
        return ULibTaskReturnCode_NotFound;
    }
    return ULibTaskReturnCode_Done;
}

// @brief 从任务列表运行任务
// @param taskList 任务列表
ULibTaskReturnCode ULibTaskNext(ULibTaskList *taskList)
{
    unsigned char i, j;
    for (i = 0; i < taskList->c && taskList->tp[i]->s < ULibTaskState_Active; i++)
    {
    }
    if (i >= taskList->c)
    {
        return ULibTaskReturnCode_Noop;
    }
    j = taskList->tp[i]->p;
    do
    {
        if (taskList->tp[i]->s >= ULibTaskState_Active)
        {
            taskList->tp[i]->f(taskList->tp[i]);
        }
    } while (++i < taskList->c && taskList->tp[i]->p == j);
    return ULibTaskReturnCode_Done;
}

// @brief 把任务放入定时任务列表
// @param taskList 定时任务列表
// @param taskAddr 定时任务地址
ULibTaskReturnCode ULibTMRTaskPut(ULibTMRTaskList *TMRTaskList, ULibTMRTaskAddr TMRTaskAddr)
{
    short i;
    if (TMRTaskList->c >= ULibTMRTaskList_MaxCount)
    {
        return ULibTaskReturnCode_Full;
    }
    if (TMRTaskAddr->c)
    {
        TMRTaskAddr->_c = TMRTaskAddr->c;
    }
    for (i = TMRTaskList->c - 1; i >= 0 && TMRTaskList->tp[i]->_c > TMRTaskAddr->_c; i--)
    {
        TMRTaskList->tp[i + 1] = TMRTaskList->tp[i];
    }
    TMRTaskList->tp[i + 1] = TMRTaskAddr;
    TMRTaskList->c++;
    return ULibTaskReturnCode_Done;
}

// @brief 把任务弹出定时任务列表
// @param taskList 定时任务列表
// @param taskAddr 定时任务地址
ULibTaskReturnCode ULibTMRTaskPop(ULibTMRTaskList *TMRTaskList, ULibTMRTaskAddr TMRTaskAddr)
{
    if (TMRTaskList->c == 0)
    {
        return ULibTaskReturnCode_Empty;
    }
    for (TMRTaskList->temp[0] = 0, TMRTaskList->temp[1] = 0; TMRTaskList->temp[0] < TMRTaskList->c; TMRTaskList->temp[0]++)
    {
        if (TMRTaskList->temp[1])
        {
            TMRTaskList->tp[TMRTaskList->temp[0]] = TMRTaskList->tp[TMRTaskList->temp[0] + 1];
        }
        else if (TMRTaskList->tp[TMRTaskList->temp[0]] == TMRTaskAddr)
        {
            TMRTaskList->tp[TMRTaskList->temp[0]] = TMRTaskList->tp[TMRTaskList->temp[0] + 1];
            TMRTaskList->c--;
            TMRTaskList->temp[1] = 1;
        }
    }
    if (!TMRTaskList->temp[1])
    {
        return ULibTaskReturnCode_NotFound;
    }
    return ULibTaskReturnCode_Done;
}

// @brief 更新定时任务周期
// @param taskList 定时任务列表
// @param TMRCycle 走过的周期
ULibTaskReturnCode ULibTMRTaskPlus(ULibTMRTaskList *TMRTaskList, ULibTMRCycle TMRCycle)
{
    TMRTaskList->_c += TMRCycle;
    return ULibTaskReturnCode_Done;
}

// @brief 从定时任务列表激活任务
// @param taskList 定时任务列表
ULibTaskReturnCode ULibTMRTaskCheck(ULibTMRTaskList *TMRTaskList)
{
    for (TMRTaskList->temp[2] = 0; TMRTaskList->temp[2] < TMRTaskList->c; TMRTaskList->temp[2]++)
    {
        if (TMRTaskList->tp[TMRTaskList->temp[2]]->_c <= TMRTaskList->_c)
        {
            TMRTaskList->tp[TMRTaskList->temp[2]]->tp->bp = TMRTaskList->tp[TMRTaskList->temp[2]]->bp;
            TMRTaskList->tp[TMRTaskList->temp[2]]->tp->s = ULibTaskState_Active;
            if (TMRTaskList->tp[TMRTaskList->temp[2]]->c)
            {
                TMRTaskList->tp[TMRTaskList->temp[2]]->_c = TMRTaskList->tp[TMRTaskList->temp[2]]->c;
            }
            else
            {
                ULibTMRTaskPop(TMRTaskList, TMRTaskList->tp[TMRTaskList->temp[2]]);
            }
        }
        else
        {
            TMRTaskList->tp[TMRTaskList->temp[2]]->_c -= TMRTaskList->_c;
        }
    }
    TMRTaskList->_c = 0;
    return ULibTaskReturnCode_Done;
}