#include "ULibSTC32.h"
#include "PWMB.h"

ULibTaskList taskList;
ULibTask TaskServoLibIO;

unsigned short PWMP, PWMM0, PWMM1;

#define ServoTh 100

#define TaskServoLibIOTail_ISP 0x80
#define TaskServoLibIOTail_Beep 0x40
#define TaskServoLibIOFeedBack_Reset 0x01
#define TaskServoLibIOFeedBack_Pause 0x02

typedef struct _ServoLibIOBuf
{
    float Servo[2];
    unsigned char tail;
    unsigned char feedBack;
} ServoLibIOBuf;

ServoLibIOBuf xdata slioBuf;

#define TaskServoLibIOState_WaitCmd 0
#define TaskServoLibIOState_SyncSta 1
void TaskServoLibIOFunc(ULibTaskAddr taskAddr)
{
    switch (taskAddr->bp)
    {
    case TaskServoLibIOState_SyncSta:
        if (slioBuf.tail)
        {
            if (slioBuf.tail & TaskServoLibIOTail_ISP)
            {
                STCReset(1);
            }
            else if (slioBuf.tail & TaskServoLibIOTail_Beep)
            {

                P06 = 1;
            }
            else
            {
                P06 = 0;
            }
            if (slioBuf.Servo[0] > ServoTh)
            {
                slioBuf.Servo[0] = ServoTh;
            }
            else if (slioBuf.Servo[0] < 0)
            {
                slioBuf.Servo[0] = 0;
            }
            if (slioBuf.Servo[1] > ServoTh)
            {
                slioBuf.Servo[1] = ServoTh;
            }
            else if (slioBuf.Servo[1] < 0)
            {
                slioBuf.Servo[1] = 0;
            }
            PWMM0 = PWMP * (0.2 + 0.2 / ServoTh * slioBuf.Servo[0]);
            PWMBValue(0, &PWMM0);
            PWMM1 = PWMP * (0.2 + 0.2 / ServoTh * slioBuf.Servo[1]);
            PWMBValue(1, &PWMM1);
        }
        slioBuf.feedBack = P0 >> 4;
        URTSend(taskAddr, TaskServoLibIOState_WaitCmd, 1, &slioBuf.feedBack, 1);
        break;
    case TaskServoLibIOState_WaitCmd:
    default:
        UR1RTaskAddr = AddrNull;
        URTRecv(taskAddr, TaskServoLibIOState_SyncSta, 1, (DataAddr)&slioBuf, 9);
        break;
    }
}

void main(void)
{
    P06 = 0;
    STC32Init();
    PWMBInit(200);
    PWMBNormal(0);
    PWMBNormal(1);
    PWMP = PWMBPeroid();
    GIOInit(GIOMode_IO, 0, 0xff);
    URTInit(0, URTMode_8bit, 1, 2);
    TMRInit(0, TMRMode_BaudGenerator, 2, 115200);

    TaskServoLibIO.p = 100;
    TaskServoLibIO.f = TaskServoLibIOFunc;
    TaskServoLibIO.s = ULibTaskState_Active;
    ULibTaskPut(&taskList, &TaskServoLibIO);

    while (1)
    {
        ULibTaskNext(&taskList);
    }
}